; 2600 Radial Pong
; (c) 2002 by Jeffry Johnston, jeffryj@azstarnet.com
; All rights reserved
; Based on Java Radial Pong by Andrzej Kapolka
; Assemble with TASM

; 2600 MEMORY ADDRESSES
; =====================
; screen control 
VSYNC   .EQU    $00     ; (w) start/end vertical sync (3 lines)
VBLANK  .EQU    $01     ; (w) start/end vertical blank (37 lines)
HBLANK  .EQU    $02     ; (w) wait for horizontal blank
RSYNC   .EQU    $03     ; (w) reset sync

; player/missile graphics
NUSIZ0  .EQU    $04     ; (w) width & clones of P0/M0 (player 0/missile 0)
NUSIZ1  .EQU    $05     ; (w) width & clones of P1/M1
COLPM0  .EQU    $06     ; (w) color of P0/M0
COLPM1  .EQU    $07     ; (w) color of P1/M1
COLPF   .EQU    $08     ; (w) color of PF/BL (playfield/ball)
COLBK   .EQU    $09     ; (w) color of BK (background)
CTRLPBC .EQU    $0A     ; (w) control playfield/ball/collisions
                        ; D0: reflect PF
                        ; D1: PF color override P0 left/P1 right
                        ; D2: playfield priority (sprites go under PF)
                        ; D5/D4: 0=BL width 1,1=width 2,2=width 4,3=width 8
MIRP0   .EQU    $0B     ; (w) mirror P0
MIRP1   .EQU    $0C     ; (w) mirror P1
PF0     .EQU    $0D     ; (w) playfield left (4 bits)
PF1     .EQU    $0E     ; (w) playfield middle (8 bits)
PF2     .EQU    $0F     ; (w) playfield right (8 bits)
GRP0    .EQU    $1B     ; (w) graphics for P0
GRP1    .EQU    $1C     ; (w) graphics for P0
ENAM0   .EQU    $1D     ; (w) enable M0
ENAM1   .EQU    $1E     ; (w) enable M1
ENABL   .EQU    $1F     ; (w) enable BL
HMP0    .EQU    $20     ; (w) horizontal motion P0
HMP1    .EQU    $21     ; (w) horizontal motion P1
HMM0    .EQU    $22     ; (w) horizontal motion M0
HMM1    .EQU    $23     ; (w) horizontal motion M1
HMBL    .EQU    $24     ; (w) horizontal motion BL
HMDO    .EQU    $2A     ; (w) do horizontal motion
VDELP0  .EQU    $25     ; (w) vertical delay P0
VDELP1  .EQU    $26     ; (w) vertical delay P1
VDELBL  .EQU    $27     ; (w) vertical delay BL
RESP0   .EQU    $10     ; (w) reset P0
RESP1   .EQU    $11     ; (w) reset P1
RESM0   .EQU    $12     ; (w) reset M0
RESM1   .EQU    $13     ; (w) reset M1
RESBL   .EQU    $14     ; (w) reset BL
RESMP0  .EQU    $28     ; (w) reset M0 to P0
RESMP1  .EQU    $29     ; (w) reset M1 to P1
RESHM   .EQU    $2B     ; (w) reset horizontal motion
RESCX   .EQU    $2C     ; (w) reset collisions
CXM0P   .EQU    $00     ; (r) M0/Px collision
CXM1P   .EQU    $01     ; (r) M1/Px collision
CXP0FB  .EQU    $02     ; (r) P0/PF, P0/BL collision
CXP1FB  .EQU    $03     ; (r) P1/PF, P1/BL collision
CXM0FB  .EQU    $04     ; (r) M0/PF, M0/BL collision
CXM1FB  .EQU    $05     ; (r) M1/PF, M1/BL collision
CXBLPF  .EQU    $06     ; (r) BL/PF collision
CXPPMM  .EQU    $07     ; (r) P0/P1, M0/M1 collision

; sound 
AUDC0   .EQU    $15     ; (w) audio control 0
AUDC1   .EQU    $16     ; (w) audio control 1
AUDF0   .EQU    $17     ; (w) audio frequency 0
AUDF1   .EQU    $18     ; (w) audio frequency 1
AUDV0   .EQU    $19     ; (w) audio volume 0
AUDV1   .EQU    $1A     ; (w) audio volume 1

; I/O
INPT0   .EQU    $08     ; (r) input pot 0
INPT1   .EQU    $09     ; (r) input pot 1
INPT2   .EQU    $0A     ; (r) input pot 2
INPT3   .EQU    $0B     ; (r) input pot 3
INTR0   .EQU    $0C     ; (r) input trigger 0
INTR1   .EQU    $0D     ; (r) input trigger 1
PORTA   .EQU    $0280   ; (r/w) port A data (joysticks)
PORTADR .EQU    $0281   ; (r?/w) port A direction
PORTB   .EQU    $0282   ; (r/w?) port B data (switches)
PORTBDR .EQU    $0283   ; (r?/w) port B direction

; timer (also at $0380-$0397)
TIMER   .EQU    $0284   ; (r/w?) timer output
TIM1C   .EQU    $0294   ; (r?/w) set 1 clock interval
TIM8C   .EQU    $0295   ; (r?/w) set 1 clock interval
TIM64C  .EQU    $0296   ; (r?/w) set 64 clock interval
TIM1KC  .EQU    $0297   ; (r?/w) set 1024 clock interval

; RAM
RAM     .EQU    $80     ; RAM $80-$FF (also at $0180-$01FF)

.ORG    $F000           ; ROM $F000-$FFFF
;---------------------------------------------------------------------
; GAME EQUATES
;---------------------------------------------------------------------
POSXP0  .EQU    $80
POSYP0  .EQU    $81
ARCP0   .EQU    $82
SCOREP0 .EQU    $83
P0      .EQU    $88     ; $88-$8F

POSXP1  .EQU    $90
POSYP1  .EQU    $91
ARCP1   .EQU    $92
SCOREP1 .EQU    $93
P1      .EQU    $98     ; $98-$9F

BALLX   .EQU    $A0
BALLY   .EQU    $A1
TEMPX   .EQU    $A2
TEMPY   .EQU    $A3
TEMP    .EQU    $A4
TEMP2   .EQU    $A5

SIN     .EQU    $FB00   ; $FB00-$FBFF
COS     .EQU    $FC00   ; $FC00-$FCFF
LEFT    .EQU    $FD00   ; $FD00-$FDFF
TOP     .EQU    $FE00   ; $FE00-$FEFF
POWER2  .EQU    $FF00   ; $FF00-$FF0F

;---------------------------------------------------------------------
; START
;---------------------------------------------------------------------
START   SEI                     ; Disable interrupts
        CLD                     ; Clear decimal mode
        LDX     #$FF     
        TXS                     ; Set up stack at $FF 
        LDA     #$00
CLRRAM  STA     $00,X           ; Clear RAM (except VSYNC)
        DEX     
        BNE     CLRRAM
        JSR     INIT
MAIN    JSR     SYNC            ; Main game loop   
        JSR     GAME
        JSR     KERNEL
        JSR     OVERSCN
        JMP     MAIN

;---------------------------------------------------------------------
; INIT
;---------------------------------------------------------------------
INIT    LDA     #$00
        STA     SCOREP0
        STA     SCOREP1
        STA     ARCP0
        STA     ARCP1
        LDA     #$0F
        STA     COLPM0
        STA     COLPM1
        STA     COLPF
        LDA     LEFT+$00  
        STA     POSXP0
        STA     POSXP1
        RTS

;---------------------------------------------------------------------
; SYNC -- 3 lines of VSYNC
;---------------------------------------------------------------------
SYNC    LDA     #$02
        STA     HBLANK
        STA     HBLANK
        STA     HBLANK
        STA     VSYNC   ; Begin vertical sync
        STA     HBLANK  ; First line of VSYNC
        STA     HBLANK  ; Second line of VSYNC
        LDA     #44     ; 37 VBLANK*76 cycles-14 waste/64 time=44 round down
        STA     TIM64C  ; set timer to activate during last line of VBLANK
        LDA     #$00
        STA     HBLANK  ; Third line of VSYNC
        STA     VSYNC   ; End vertical sync period
        RTS

;---------------------------------------------------------------------
; GAME -- process joysticks and update paddle images
;---------------------------------------------------------------------
GAME
;handle collisions
        LDA     CXP0FB          ; Get P0/BL collision info
        AND     #%11000000
        BEQ     NOHITP0         ; hit detected? 
        LDA     #$D8
        STA     COLBK           ; yes, set background color
NOHITP0 LDA     CXP1FB          ; Get P1/BL collision info
        AND     #%11000000
        BEQ     NOHITP1         ; hit detected? 
        LDA     #$58
        STA     COLBK           ; yes, set background color
NOHITP1 STA     RESCX           ; Reset collision registers
;move ball
        INC     BALLY
        LDA     #$10
        STA     HMBL            ; update BL position
        LDA     #%00010001
        STA     CTRLPBC         ; Set BL width
;move P0/P1 with joystick
        LDA     PORTA           ; Read joystick
        BMI     NR1     
;Right
        DEC     ARCP0           ; update P0 position 
NR1     ROL     A
        BMI     NL1
;Left
        INC     ARCP0           ; update P0 position
NL1     ROL     A
        BMI     ND1
;Down
        INC     ARCP1           ; update P1 position
ND1     ROL     A
        BMI     NU1
;Up
        DEC     ARCP1           ; update P1 position
NU1     JSR     SPRITE          ; Update player sprites
        RTS

;---------------------------------------------------------------------
; SPRITE -- create P0/P1 sprites from circle data
;---------------------------------------------------------------------
SPRITE:
;set up P0
        LDX     ARCP0
        LDA     TOP,X           ; get TOP coord
        STA     POSYP0
        LDA     LEFT,X          ; get LEFT coord
        STA     TEMPX
        LDA     #$00
        LDY     #$07
        STY     TEMP2
;clear the sprite
CLEARP0 STA     P0,Y
        DEY
        BPL     CLEARP0
;fill in the sprite
FILLP0  LDA     COS,X           ; A=Y coord
        SEC
        SBC     POSYP0          ; A=A-POSYP0 (sprite row 0-7)
        STA     TEMP            ; TEMP=A
        LDA     SIN,X           ; A=X coord
        SEC
        SBC     TEMPX           ; A=A-TEMPX (sprite column 0-7)
        TAY                     ; Y=A
        LDA     POWER2,Y        ; A=2^Y 
        LDY     TEMP            ; Y=TEMP
        ORA     P0,Y            ; A=A OR P0(Y)
        STA     P0,Y            ; P0(Y)=A
        DEX
        DEC     TEMP2
        BPL     FILLP0
        LDA     POSXP0
        SEC
        SBC     TEMPX           ; A=POSXP0-TEMPX
        ASL     A               ; move difference into upper nibble
        ASL     A               
        ASL     A
        ASL     A
        STA     HMP0            ; store new horizontal motion value
        LDA     TEMPX
        STA     POSXP0          ; update POSXP0=TEMPX
;set up P1
        LDX     ARCP1
        LDA     TOP,X           ; get TOP coord
        STA     POSYP1
        LDA     LEFT,X          ; get LEFT coord
        STA     TEMPX
        LDA     #$00
        LDY     #$07
        STY     TEMP2
;clear the sprite
CLEARP1 STA     P1,Y
        DEY
        BPL     CLEARP1
;fill in the sprite
FILLP1  LDA     COS,X           ; A=Y coord
        SEC
        SBC     POSYP1          ; A=A-POSYP1 (sprite row 0-7)
        STA     TEMP            ; TEMP=A
        LDA     SIN,X           ; A=X coord
        SEC
        SBC     TEMPX           ; A=A-TEMPX (sprite column 0-7)
        TAY                     ; Y=A
        LDA     POWER2,Y        ; A=2^Y 
        LDY     TEMP            ; Y=TEMP
        ORA     P1,Y            ; A=A OR P1(Y)
        STA     P1,Y            ; P1(Y)=A
        DEX
        DEC     TEMP2
        BPL     FILLP1
        LDA     POSXP1
        SEC
        SBC     TEMPX           ; A=POSXP1-TEMPX
        ASL     A               ; move difference into upper nibble
        ASL     A
        ASL     A
        ASL     A
        STA     HMP1            ; store new horizontal motion
        LDA     TEMPX
        STA     POSXP1          ; update POSXP1=TEMPX
        RTS

;---------------------------------------------------------------------
; KERNEL -- 37 lines VBLANK then 192 lines of KERNEL
;---------------------------------------------------------------------
KERNEL  LDA     TIMER   ; Get timer value
        BNE     KERNEL  ; If time isn't up then waste more time
        STA     HBLANK  ; End last line of VBLANK
        STA     HMDO    ; DON'T FORGET TO HIT HMDO or the object won't move
        STA     VBLANK  ; End VBLANK
;kernel start
        LDY     #$00    ; Y will store the scanline value
NEXTLNE STA     HBLANK  ; End the current scanline
;player 1
        TYA             ; A=Y
        SEC
        SBC     POSYP1  ; A=A-POSYP1
        CMP     #8      ; Height of sprite in pixels            
        BCS     NOP1    
        TAX
        LDA     P1,X    ; Value to draw sprite (on GRP1)
        JMP     DRAWP1
NOP1    LDA     #$00    ; Value to blank out sprite (on GRP1)
DRAWP1  STA     GRP1    ; Draw sprite (pattern depends on A)
;end scanline again (for 96 line resolution)
        STA     HBLANK                      
;player 0
        TYA             ; A=Y
        SEC
        SBC     POSYP0  ; A=A-POSYP0
        CMP     #8      ; Height of sprite in pixels            
        BCS     NOP0    
        TAX
        LDA     P0,X    ; Value to draw sprite (on GRP0)
        JMP     DRAWP0
NOP0    LDA     #$00    ; Value to blank out sprite (on GRP0)
DRAWP0  STA     GRP0    ; Draw sprite (pattern depends on A)
;ball
        LDX     #$00    ; Value to disable sprite (on ENABL)
        TYA             ; A=Y
        SEC
        SBC     BALLY   ; A=A-BALLY
        CMP     #2      ; Height of ball in pixels
        BCS     NOBL
        LDX     #$02    ; Value to enable sprite (on ENABL)
NOBL    STX     ENABL   ; Enable/disable sprite (depending on X)
;kernel continued...
        INY             ; Count the line
        CPY     #96     ; out of lines?
        BNE     NEXTLNE ; if no, do next line
        RTS             ; yes

;---------------------------------------------------------------------
; OVERSCN -- 30 lines of overscan
;---------------------------------------------------------------------
OVERSCN LDX     #30
WASTE   STA     HBLANK
        DEX
        BNE     WASTE
        RTS

;---------------------------------------------------------------------
; TABLES & DATA
;---------------------------------------------------------------------
.ORG    $FB00   ; sin, cos, left, top
#INCLUDE        SINCOS.INC
.ORG    $FF00   ; POWER2
        .BYTE   $80,$40,$20,$10,$08,$04,$02,$01
.ORG    $FFFC
        .WORD   START
        .WORD   START
.END
